package cz.muni.fi.group05.room03.ui.table.util;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

// COPIED FROM http://www.camick.com/java/source/TableColumnManager.java
/**
 *  The TableColumnManager can be used to manage TableColumns. It will give the
 *  user the ability to hide columns and then reshow them in their last viewed
 *  position. This functionality is supported by a popup menu added to the
 *  table header of the table. The TableColumnModel is still used to control
 *  the view for the table. The manager will inovoke the appropriate methods
 *  of the TableColumnModel to hide/show columns as required.
 *
 */
public class TableColumnManager {

    private final JTable table;
    private TableColumnModel tcm;

    /**
     *  Create a TableColumnManager for a table.
     *
     *  @param table the table whose TableColumns will managed.
     */
    public TableColumnManager(JTable table) {
        this.table = table;
        reset();
    }

    /**
     *  Reset the TableColumnManager to only manage the TableColumns that are
     *  currently visible in the table.
     *
     *  Generally this method should only be invoked by the TableColumnManager
     *  when the TableModel of the table is changed.
     */
    public void reset() {
        tcm = table.getColumnModel();

        //  Keep a duplicate TableColumns for managing hidden TableColumns
        int count = tcm.getColumnCount();
        List<TableColumn> allColumns = new ArrayList<>(count);

        for (int i = 0; i < count; i++) {
            allColumns.add(tcm.getColumn(i));
        }
    }

    /**
     *  Hide a column from view in the table.
     *
     *  @param  modelColumn  the column index from the TableModel
     *                       of the column to be removed
     */
    public void hideColumn(int modelColumn) {
        int viewColumn = table.convertColumnIndexToView(modelColumn);
        if (viewColumn != -1) {
            TableColumn column = tcm.getColumn(viewColumn);
            hideColumn(column);
        }
    }

    /**
     *  Hide a column from view in the table.
     *
     *  @param  columnName   the column name of the column to be removed
     */
    public void hideColumn(Object columnName) {
        if (columnName == null) {
            return;
        }
        for (int i = 0; i < tcm.getColumnCount(); i++) {
            TableColumn column = tcm.getColumn(i);
            if (columnName.equals(column.getHeaderValue())) {
                hideColumn(column);
                break;
            }
        }
    }

    /**
     *  Hide a column from view in the table.
     *
     *  @param  column  the TableColumn to be removed from the
     *                  TableColumnModel of the table
     */
    public void hideColumn(TableColumn column) {
        if (tcm.getColumnCount() == 1) {
            return;
        }
        //  Ignore changes to the TableColumnModel made by the TableColumnManager
        tcm.removeColumn(column);
    }
}
